@@ -1,4 +1,4 @@
-# $Id: TwoWay.pm 2220 2007-08-19 00:59:33Z rcaputo $
+# $Id: TwoWay.pm,v 1.17 2004/12/02 00:20:58 apocal Exp $
 
 # Portable two-way pipe creation, trying as many different methods as
 # we can.
@@ -8,7 +8,7 @@ package POE::Pipe::TwoWay;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2220 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my@r=(q$Revision: 1.17 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 use Symbol qw(gensym);
 use IO::Socket qw( AF_UNIX SOCK_STREAM PF_UNSPEC );
@@ -40,14 +40,14 @@ sub new {
       );
   }
 
-  while (my $try_type = $self->_get_next_preference()) {
+  while (my $try_type = $self->get_next_preference()) {
     return ($a_read, $a_write, $b_read, $b_write) if
       $self->_try_type(
         $try_type,
         \$a_read, \$a_write,
         \$b_read, \$b_write
       );
-    $self->_shift_preference();
+    $self->shift_preference();
   }
 
   # There's nothing left to try.
@@ -117,7 +117,7 @@ sub _try_type {
   # Try a pair of plain INET sockets.
   if ($type eq "inet") {
     eval {
-      ($$a_read, $$b_read) = $self->_make_socket();
+      ($$a_read, $$b_read) = $self->make_socket();
     };
 
     # Sockets failed.
@@ -185,16 +185,6 @@ And now you have an unbuffered pipe with two read/write sides, A and
 B.  Writing to C<$a_write> passes data to C<$b_read>, and writing to
 C<$b_write> passes data to C<$a_read>.
 
-=head1 CONSTRUCTOR
-
-=over
-
-=item new
-
-  my ($a_read, $a_write, $b_read, $b_write) = POE::Pipe::TwoWay->new();
-
-=back
-
 =head1 DEBUGGING
 
 It's possible to force POE::Pipe::TwoWay to use one of its underlying
@@ -215,6 +205,3 @@ reserved.  POE::Pipe::TwoWay is free software; you may redistribute it
 and/or modify it under the same terms as Perl itself.
 
 =cut
-
-# rocco // vim: ts=2 sw=2 expandtab
-# TODO - Redocument.
