@@ -1,11 +1,11 @@
-# $Id: Session.pm,v 1.106 2004/12/02 16:21:20 rcaputo Exp $
+# $Id: Session.pm,v 1.113 2005/07/25 02:12:25 rcaputo Exp $
 
 package POE::Session;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my@r=(q$Revision: 1.106 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my@r=(q$Revision: 1.113 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 use Carp qw(carp croak);
 use Errno qw(ENOSYS);
@@ -163,6 +163,25 @@ sub import {
   *{ $package . '::CALLER_STATE' } = \&CALLER_STATE;
 }
 
+sub instantiate {
+  my $type = shift;
+
+  croak "$type requires a working Kernel"
+    unless defined $POE::Kernel::poe_kernel;
+
+  my $self =
+    bless [ { }, # SE_NAMESPACE
+            { }, # SE_OPTIONS
+            { }, # SE_STATES
+          ], $type;
+
+  if (ASSERT_STATES) {
+    $self->[SE_OPTIONS]->{+OPT_DEFAULT} = 1;
+  }
+
+  return $self;
+}
+
 sub try_alloc {
   my ($self, @args) = @_;
   # Verify that the session has a special start state, otherwise how
@@ -196,17 +215,12 @@ sub new {
   croak "sessions no longer require a kernel reference as the first parameter"
     if ((@states > 1) && (ref($states[0]) eq 'POE::Kernel'));
 
-  croak "$type requires a working Kernel"
-    unless defined $POE::Kernel::poe_kernel;
+  carp(
+    "POE::Session->new() is deprecated.  Please use create() instead.\n",
+    "See http://www.nntp.perl.org/group/perl.poe/2613 for more information."
+  );
 
-  my $self =
-    bless [ { }, # SE_NAMESPACE
-            { }, # SE_OPTIONS
-            { }, # SE_STATES
-          ], $type;
-  if (ASSERT_STATES) {
-    $self->[SE_OPTIONS]->{+OPT_DEFAULT} = 1;
-  }
+  my $self = $type->instantiate (\@states);
 
   # Scan all arguments.  It mainly expects them to be in pairs, except
   # for some, uh, exceptions.
@@ -370,17 +384,7 @@ sub create {
   }
   my %params = @params;
 
-  croak "$type requires a working Kernel"
-    unless defined $POE::Kernel::poe_kernel;
-
-  my $self =
-    bless [ { }, # SE_NAMESPACE
-            { }, # SE_OPTIONS
-            { }, # SE_STATES
-          ], $type;
-  if (ASSERT_STATES) {
-    $self->[SE_OPTIONS]->{+OPT_DEFAULT} = 1;
-  }
+  my $self = $type->instantiate(\%params);
 
   # Process _start arguments.  We try to do the right things with what
   # we're given.  If the arguments are a list reference, map its items
@@ -631,7 +635,7 @@ sub _invoke_state {
         undef,                          # unused #6
         $file,                          # caller file name
         $line,                          # caller file line
-		$fromstate,						# caller state
+    $fromstate,            # caller state
         @$etc                           # args
       );
   }
@@ -649,7 +653,7 @@ sub _invoke_state {
         undef,                          # unused #6
         $file,                          # caller file name
         $line,                          # caller file line
-		$fromstate,						# caller state
+    $fromstate,            # caller state
         @$etc                           # args
       );
 }
@@ -829,16 +833,16 @@ sub get_heap {
 # perform the necessary cleanup when they go away.  Thanks to njt for
 # steering me right on this one.
 
-my %postback_parent_id;
+my %anonevent_parent_id;
 
 # I assume that when the postback owner loses all reference to it,
 # they are done posting things back to us.  That's when the postback's
 # DESTROY is triggered, and referential integrity is maintained.
 
-sub POE::Session::Postback::DESTROY {
+sub POE::Session::AnonEvent::DESTROY {
   my $self = shift;
-  my $parent_id = delete $postback_parent_id{$self};
-  $POE::Kernel::poe_kernel->refcount_decrement( $parent_id, 'postback' );
+  my $parent_id = delete $anonevent_parent_id{$self};
+  $POE::Kernel::poe_kernel->refcount_decrement( $parent_id, 'anon_event' );
 }
 
 # Tune postbacks depending on variations in toolkit behavior.
@@ -866,10 +870,10 @@ sub postback {
   my $postback = bless sub {
     $POE::Kernel::poe_kernel->post( $id, $event, [ @etc ], [ @_ ] );
     return 0;
-  }, 'POE::Session::Postback';
+  }, 'POE::Session::AnonEvent';
 
-  $postback_parent_id{$postback} = $id;
-  $POE::Kernel::poe_kernel->refcount_increment( $id, 'postback' );
+  $anonevent_parent_id{$postback} = $id;
+  $POE::Kernel::poe_kernel->refcount_increment( $id, 'anon_event' );
 
   # Tk blesses its callbacks, so we must present one that isn't
   # blessed.  Otherwise Tk's blessing would divert our DESTROY call to
@@ -881,18 +885,24 @@ sub postback {
 
 # Create a synchronous callback closure.  The return value will be
 # passed to whatever is handed the callback.
-#
-# TODO - Should callbacks hold reference counts like postbacks do?
 
 sub callback {
   my ($self, $event, @etc) = @_;
   my $id = $POE::Kernel::poe_kernel->ID_session_to_id($self);
 
-  my $callback = sub {
-    return $POE::Kernel::poe_kernel->call( $id, $event, [ @etc ], [ @_ ] );
-  };
+  my $callback = bless sub {
+    $POE::Kernel::poe_kernel->call( $id, $event, [ @etc ], [ @_ ] );
+  }, 'POE::Session::AnonEvent';
 
-  $callback;
+  $anonevent_parent_id{$callback} = $id;
+  $POE::Kernel::poe_kernel->refcount_increment( $id, 'anon_event' );
+
+  # Tk blesses its callbacks, so we must present one that isn't
+  # blessed.  Otherwise Tk's blessing would divert our DESTROY call to
+  # its own, and that's not right.
+
+  return sub { $callback->(@_) } if USING_TK;
+  return $callback;
 }
 
 ###############################################################################
@@ -909,7 +919,8 @@ POE::Session - an event driven abstract state machine
   # Import POE::Session constants.
   use POE::Session;
 
-  # The older, more DWIMmy constructor.
+  # The older, more DWIMmy constructor, which will soon be deprecated
+  # (i.e., DON'T USE THIS).
   POE::Session->new(
 
     # Inline or coderef states.
@@ -932,20 +943,20 @@ POE::Session - an event driven abstract state machine
   POE::Session->create(
 
     # Inline or coderef states.
-    inline_states =>
-      { state_one => \&coderef_one,
-        state_two => sub { ... },
-      },
+    inline_states => {
+      state_one => \&coderef_one,
+      state_two => sub { ... },
+    },
 
     # Plain and mapped object states.
-    object_states =>
-    [ $object_one => [ 'state_three', 'state_four', 'state_five' ],
+    object_states => [
+      $object_one => [ 'state_three', 'state_four', 'state_five' ],
       $object_two => { state_nine => 'method_nine' },
     ],
 
     # Plain and mapped package states.
-    package_states =>
-    [ $package_one => [ 'state_six', 'state_seven', 'state_eight' ],
+    package_states => [
+      $package_one => [ 'state_six', 'state_seven', 'state_eight' ],
       $package_two => { state_ten => 'method_ten' },
     ],
 
@@ -1082,8 +1093,8 @@ C<inline_states> maps events names to the plain coderefs which will
 handle them.  Its value is a reference to a hash of event names and
 corresponding coderefs.
 
-  inline_states =>
-  { _start => sub { print "arg0=$_[ARG0], arg1=$_[ARG1], etc.=$_[ARG2]\n"; }
+  inline_states => {
+    _start => sub { print "arg0=$_[ARG0], arg1=$_[ARG1], etc.=$_[ARG2]\n"; }
     _stop  => \&stop_handler,
   },
 
@@ -1104,8 +1115,8 @@ form maps each event to an object method with the same name.  In this
 example, C<event_one> is handled by C<$object>'s C<event_one()>
 method.
 
-  object_states =>
-  [ $object => [ 'event_one', 'event_two' ],
+  object_states => [
+    $object => [ 'event_one', 'event_two' ],
   ];
 
 The second form associates a hashref to each object reference.  In
@@ -1114,10 +1125,11 @@ this form, the object's method names needn't match the event names
 they'll handle.  For example, C<event_four> is handled by C<$object's>
 C<handler_four()> method.
 
-  object_states =>
-  [ $object => { event_three => 'handler_three',
-                 event_four  => 'handler_four',
-               }
+  object_states => [
+    $object => {
+      event_three => 'handler_three',
+      event_four  => 'handler_four',
+    }
   ];
 
 =item options => HASHREF
@@ -1154,8 +1166,8 @@ maps each event to a package method with the same name.  In this
 example, C<event_ten> is handled by C<Package>'s C<event_ten()>
 method.
 
-  package_states =>
-  [ Package => [ 'event_ten', 'event_eleven' ],
+  package_states => [
+    Package => [ 'event_ten', 'event_eleven' ],
   ];
 
 The second form associates a hashref to each package name.  In turn,
@@ -1164,10 +1176,11 @@ form, the package's method names needn't match the event names they'll
 handle.  For example, C<event_twelve> is handled by C<Package>'s
 C<handler_twelve()> method.
 
-  package_states =>
-  [ Package => { event_twelve   => 'handler_twelve',
-                 event_thirteen => 'handler_thirteen',
-               }
+  package_states => [
+    Package => {
+      event_twelve   => 'handler_twelve',
+      event_thirteen => 'handler_thirteen',
+    }
   ];
 
 =back
@@ -1265,8 +1278,7 @@ B<previous> value of OPTION_NAME.
 The final form sets several options, returning a hashref containing
 pairs of option names and their B<previous> values.
 
-  my $old_values = $_[SESSION]->option(
-    trace => $new_trace_value,
+  my $old_values = $_[SESSION]->option( trace => $new_trace_value,
     debug => $new_debug_value,
   );
   print "Old option values:\n";
@@ -1274,139 +1286,76 @@ pairs of option names and their B<previous> values.
     print "$option = $old_value\n";
   }
 
-=item postback EVENT_NAME, PARAMETER_LIST
+=item postback EVENT_NAME, EVENT_PARAMETERS
 
-postback() creates anonymous coderefs which, when called, post
-EVENT_NAME events back to the session whose postback() method was
-called.  Postbacks hold external references on the sessions they're
-created for, so they keep their sessions alive.
+=item callback EVENT_NAME, EVENT_PARAMETERS
 
-The EVENT_NAME event includes two fields, both of which are list
-references.  C<ARG0> contains a reference to the PARAMETER_LIST passed
-to C<postback()>.  This is the "request" field.  C<ARG1> holds a
-reference to the parameters passed to the coderef when it's called.
-That's the "response" field.
+postback() and callback() create anonymous coderefs that may be used
+as callbacks for other libraries.  A contrived example:
 
-This creates a Tk button that posts an "ev_counters_begin" event to
-C<$session> whenever it's pressed.
+  my $postback = $session->postback( event_one => 1, 2, 3 );
+  my $callback = $session->callback( event_two => 5, 6, 7 );
 
-  $poe_tk_main_window->Button
-    ( -text    => 'Begin Slow and Fast Alarm Counters',
-      -command => $session->postback( 'ev_counters_begin' )
-    )->pack;
+  use File::Find;
+  find( $callback, @directories_to_search );
 
-C<postback()> works wherever a callback does.  Another good use of
-postbacks is for request/response protocols between sessions.  For
-example, a client session will post an event to a server session.  The
-client may include a postback as part of its request event, or the
-server may build a postback based on C<$_[SENDER]> and an event name
-either pre-arranged or provided by the client.
+  $poe_main_window->Button(
+    -text    => 'Begin Slow and Fast Alarm Counters',
+    -command => $postback,
+  )->pack;
 
-Since C<postback()> is a Session method, you can call it on
-C<$_[SESSION]> to create a postback for the current session.  In this
-case, the client gives its postback to the server, and the server
-would call the postback to return a response event.
+When called, postbacks and callbacks fire POE events.  Postbacks use
+$kernel->post(), and callbacks use $kernel->call().  See POE::Kernel
+for post() and call() documentation.
 
-  # This code is in a client session.  SESSION is this session, so it
-  # refers to the client.
-  my $client_postback = $_[SESSION]->postback( reply_event_name => $data );
+Each takes an EVENT_NAME, which is the name of the event to fire when
+called.  Any other EVENT_PARAMETERS are passed to the event's handler
+as a list reference in ARG0.
 
-The other case is where the server creates a postback to respond to a
-client.  Here, it calls C<postback()> on C<$_[SENDER]> to create a
-postback that will respond to the request's sender.
+Calling C<<$postback->("a", "b", "c")>> results in event_one's handler
+being called with the following arguments:
 
-  # This code is in a server session.  SENDER is the session that sent
-  # a request event: the client session.
-  my $client_postback = $_[SENDER]->postback( reply_event_name => $data );
-
-In the following code snippets, Servlet is a session that acts like a
-tiny daemon.  It receives requests from "client" sessions, performs
-some long-running task, and eventually posts responses back.  Client
-sends requests to Servlet and eventually receives its responses.
-
-  # Aliases are a common way for daemon sessions to advertise
-  # themselves.  They also provide convenient targets for posted
-  # requests.  Part of Servlet's initialization is setting its alias.
-
-  sub Servlet::_start {
-    ...;
-    $_[KERNEL]->alias_set( 'server' );
+  sub handle_event_one {
+    my $passed_through = $_[ARG0];  # [ 1, 2, 3 ]
+    my $passed_back    = $_[ARG1];  # [ "a", "b", "c" ]
   }
 
-  # This function accepts a request event.  It creates a postback
-  # based on the sender's information, and it saves the postback until
-  # it's ready to be used.  Postbacks keep their sessions alive, so
-  # this also ensures that the client will wait for a response.
+Calling C<<$callback->("m", "n", "o")>> does the same:
 
-  sub Servlet::accept_request_event {
-    my ($heap, $sender, $reply_to, @request_args) =
-      @_[HEAP, SENDER, ARG0, ARG1..$#_];
-
-    # Set the request in motion based on @request_args.  This may take
-    # a while.
-
-    ...;
-
-    # Create a postback, and hold onto it so we'll have a way to
-    # respond back to the client session when the request has
-    # finished.
-
-    $heap->{postback}->{$sender} =
-      $sender->postback( $reply_to, @request_args );
+  sub handle_event_two {
+    my $passed_through = $_[ARG0];  # [ 5, 6, 7 ]
+    my $passed_back    = $_[ARG1];  # [ "m", "n", "o" ]
   }
 
-  # When the server is ready to respond, it retrieves the postback and
-  # calls it with the response's values.  The postback acts like a
-  # "wormhole" back to the client session.  Letting the postback fall
-  # out of scope destroys it, so it will stop keeping the session
-  # alive.  The response event, however, will take up where the
-  # postback left off, so the client will still linger at least as
-  # long as it takes to receive its response.
+Therefore you can use ARG0 to pass state through a callback, while
+ARG1 contains information provided by the external library to its
+callbacks.
 
-  sub Servlet::ready_to_respond {
-    my ($heap, $sender, @response_values) = @_[HEAP, ARG0, ARG1..$#_];
+Postbacks and callbacks use reference counts to keep the sessions they
+are called upon alive.  This prevents sessions from disappearing while
+other code expects them to handle callbacks.  The Tk Button code above
+is an example of this in action.  The session will not stop until the
+widget releases its postback.
 
-    my $postback = delete $heap->{postback}->{$sender};
-    $postback->( @response_values );
-  }
+The difference between postback() and callback() is subtle but can
+cause all manner of grief if you are not aware of it.  Postback
+handlers are not called right away since they are triggered by events
+posted through the queue.  Callback handlers are invoked immediately
+since they are triggered by call().
 
-  # This is the client's side of the transaction.  Here it posts a
-  # request to the "server" alias.
+Some libraries expect their callbacks to be invoked immediately.  They
+may go so far as to set up global variables for the duration of the
+callback.  File::Find is such a library.  Each callback receives a new
+filename in $_.  Delaying these callbacks until later means that $_
+will not contain expected values.  It is necessary to use callback()
+in these cases.
 
-  sub Client::request {
-    my $kernel = $_[KERNEL];
+Most libraries pass state to their callbacks as parameters.
+Generally, postback() is the way to go.
 
-    # Assemble a request for the server.
-    my @request = ( 1, 2, 3 );
-
-    # Post the request to the server.
-    $kernel->post( server => accept_request_event => reply_to => @request );
-  }
-
-  # Here's where the client receives its response.  Postback events
-  # have two parameters: a request block and a response block.  Both
-  # are array references containing the parameters given to the
-  # postback at construction time and at use time, respectively.
-
-  sub Client::reply_to {
-    my ($session, $request, $response) = @_[SESSION, ARG0, ARG1];
-
-    print "Session ", $session->ID, " requested: @$request\n";
-    print "Session ", $session->ID, " received : @$response\n";
-  }
-
-=item callback EVENT_NAME, PARAMETER_LIST
-
-callback() creates anonymous coderefs just like postback(), but
-instead of using $poe_kernel->post() it uses $poe_kernel->call() so
-that the event occurs synchronously, i.e. without any delay.  This is
-helpful when the data you're dealing with can change immediately after
-the callback is called.  By the time a postback() event is dispatched,
-the underlying data is long gone.
-
-callback() may also be used when your callback needs to return a
-value.  For instance, GUIs often expect a true value from a callback
-to indicate an event was handled, and a false value otherwise.
+Since postback() and callback() are Session methods, they may be
+called on $_[SESSION] or $_[SENDER], depending on particular needs.
+There are usually better ways to interact between sessions, however.
 
 =item get_heap
 
@@ -1421,7 +1370,9 @@ this:
 
   sub put_stuff {
     my @stuff_to_put = @_;
-    $poe_kernel->get_active_session()->get_heap()->{wheel}->put(@stuff);
+    $poe_kernel->get_active_session()->get_heap()->{wheel}->put(
+      @stuff_to_put
+    );
   }
 
   sub some_state {
@@ -1586,14 +1537,15 @@ C<STATE> contains the event name that invoked a state.  This is useful
 in cases where a single state handles several different events.
 
   sub some_state {
-    print( "some_state in session ", $_[SESSION]-ID,
-           " was invoked as ", $_[STATE], "\n"
-         );
+    print(
+      "some_state in session ", $_[SESSION]-ID,
+      " was invoked as ", $_[STATE], "\n"
+    );
   }
 
   POE::Session->create(
-    inline_states =>
-    { one => \&some_state,
+    inline_states => {
+      one => \&some_state,
       two => \&some_state,
       six => \&some_state,
       ten => \&some_state,
@@ -1606,8 +1558,8 @@ in cases where a single state handles several different events.
 
 =item CALLER_STATE
 
-    my ($caller_file, $caller_line, $caller_state) =
-		@_[CALLER_FILE,CALLER_LINE,CALLER_STATE];
+  my ($caller_file, $caller_line, $caller_state) =
+    @_[CALLER_FILE,  CALLER_LINE,  CALLER_STATE];
 
 The file, line number, and state from which this state was called.
 
@@ -1771,9 +1723,6 @@ States are always evaluated in a scalar context.  States that must
 return more than one value should therefore return them as a reference
 to something bigger.
 
-Signal handlers' return values are significant.  L<POE::Kernel>'s
-"Signal Watcher Methods" sections covers this is detail.
-
 States may not return references to objects in the "POE" namespace.
 The Kernel will stringify these references to prevent them from
 lingering and breaking its own garbage collection.
